#!/usr/bin/perl

# program to generate a bunch of orders and cancels.

use strict;

my @data;
my @alphabet = 'A'..'Z';
for(1..15){
    my $o = {};
    map { $o->{symbol}.=$alphabet[ int(rand($#alphabet + 1)) ] } 0..int(1+rand(2));
    $o->{lastprice} = 10+rand(150);     # 
    $o->{volatility} = $o->{lastprice}*0.001;   # price volatility.
    $o->{volume} = 1000+rand(1000);
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year += 1900;
    $mon++;
    $o->{tdate} = sprintf("%04d%02d%02d",$year,$mon,$mday);
    push @data, $o;
}

open my $ordr,">ordr.csv" or die $!;
open my $cxl,">cxl.csv" or die $!;

for my $stock (@data){
    my $samples = $stock->{volume};
    my $timticks = int(9.5 * 60 * 60 * 10000 + rand(10000));  # ~9:30 AM
    my $etimticks = int(16 * 60 * 60 * 10000 + rand(10000));  # ~4:00 PM
    my $tickinc = ($etimticks - $timticks) / $samples;
    my %ordrs;
    my @ordrs;
    my $ordrid = 1;
    my $seq = 0;
     
    while($timticks < $etimticks){
        $timticks += $tickinc + rand(100);
        $seq++;

        my $tim = int(int($timticks/10000)/3600)*10000 + 
            (int(int($timticks/10000)/60)%60)*100 + 
            int(int($timticks/10000)%60) + (int($timticks % 10000)/10000);
        
        my $tim = sprintf("%06.2f",$tim);

        if(rand() < 0.7){   # generate an order.
            my $o = {
                tdate => $stock->{tdate},
                symbol => $stock->{symbol},
                tim => $tim,
                price => $stock->{lastprice} + $stock->{volatility}*(rand(2)-1),
                qty => 100 + int(rand(50))*100,
                oid => 'A'.$ordrid++,
                seq => $seq++,
            };
            $o->{price} = $o->{price} <= 0 ? 0.01 : $o->{price};
            $o->{side} = $o->{price} > $stock->{lastprice} ? 'S' : 'B';
            $stock->{lastprice} = $o->{price};
            $ordrs{$o->{oid}} = $o;
            push @ordrs,$o;
            $o->{ordrsidx} = $#ordrs;
            $o->{price} = sprintf("%.2f",$o->{price});
            print $ordr 'ORDR,'.join(",",@{$o}{qw(tdate symbol tim seq oid side price qty)}),"\n";
        } else {            # generate a cancel.
            my $o = $ordrs[ rand() < 0.8 ? -int(rand( int (($#ordrs + 1) * 0.05) )) : int(rand($#ordrs + 1)) ];
            next unless $o;
            next unless $o->{qty} > 0;
            my $cxlqty = 100 + int(rand(50))*100;
            $cxlqty = $cxlqty < $o->{qty} ? $cxlqty : $o->{qty};
            $o->{cxlqty} = $cxlqty;
            $o->{qty} = $o->{qty} - $cxlqty;
            $o->{seq} = $seq++;
            $o->{tim} = $tim;
            print $cxl 'CXL,'.join(",",@{$o}{qw(tdate symbol tim seq oid cxlqty)}),"\n";
            if($o->{qty} <= 0){
                delete $ordrs{$o->{oid}};
                splice @ordrs,$o->{ordrsidx},1;
            }
        }
    }
}

